"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RolesManager = exports.Resource = exports.Action = void 0;
const User_1 = require("./model/User");
var Action;
(function (Action) {
    Action[Action["see"] = 0] = "see";
    Action[Action["crud"] = 1] = "crud";
})(Action = exports.Action || (exports.Action = {}));
var Resource;
(function (Resource) {
    Resource[Resource["userAdmin"] = 0] = "userAdmin";
    Resource[Resource["timetable"] = 1] = "timetable";
    Resource[Resource["cms"] = 2] = "cms";
    Resource[Resource["error"] = 3] = "error";
    Resource[Resource["hardwareAdmin"] = 4] = "hardwareAdmin";
})(Resource = exports.Resource || (exports.Resource = {}));
/**
 * Define user roles
 * visitor - default role of new user, for future usage
 * user - can run timetables
 * editor - user + can create timetables
 * admin - editor + can changes users + see errors
 */
class RolesManager {
    constructor() {
        this.roles = {
            [User_1.Role.user]: {
                [Resource.timetable]: [Action.see]
            },
            [User_1.Role.editor]: {
                [Resource.timetable]: [Action.see, Action.crud],
                [Resource.cms]: [Action.see, Action.crud],
                [Resource.error]: [Action.see],
                [Resource.hardwareAdmin]: [Action.see, Action.crud]
            },
            [User_1.Role.admin]: {
                [Resource.timetable]: [Action.see, Action.crud],
                [Resource.userAdmin]: [Action.see, Action.crud],
                [Resource.cms]: [Action.see, Action.crud],
                [Resource.error]: [Action.see, Action.crud],
                [Resource.hardwareAdmin]: [Action.see, Action.crud]
            }
        };
    }
    /**
     * Is role allowed to work with resoruce.
     * @param role
     * @param action
     * @param resource
     * @returns boolean
     */
    can(role, action, resource) {
        if (this.roles[role] === undefined || !Array.isArray(this.roles[role][resource])) {
            return false;
        }
        return this.roles[role][resource].includes(action);
    }
    /**
     * Has role permissions to more actions on resource
     * @param role
     * @param actionMore
     * @param resource
     * @returns boolean
     */
    canMore(role, actionMore, resource) {
        let allowedAllActions = true;
        actionMore.forEach((action) => {
            allowedAllActions && (allowedAllActions = this.can(role, action, resource));
        });
        return allowedAllActions;
    }
}
exports.RolesManager = RolesManager;
//# sourceMappingURL=RolesManager.js.map